/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.syncclient.spds;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

import javax.microedition.io.HttpConnection;
import javax.microedition.io.Connector;

/**
 *  Rapresents a HTTP client implementation
 *
 *  @author Fabio Maggi
 *  @version $Id: SyncMLClientImpl.java,v 1.9 2004/10/15 08:57:22 fabius Exp $
 *
 **/
public final class SyncMLClientImpl {

    // ----------------------------------------------------------------------------------------- Constants

    private static final String PROP_MICROEDITION_CONFIGURATION = "microedition.configuration"  ;
    private static final String PROP_CONTENT_LANGUAGE           = "Content-Language"            ;
    private static final String PROP_CONTENT_LENGTH             = "Content-Length"              ;
    private static final String PROP_CONTENT_TYPE               = "Content-Type"                ;
    private static final String PROP_MICROEDITION_LOCALE        = "microedition.locale"         ;
    private static final String PROP_MICROEDITION_PROFILES      = "microedition.profiles"       ;
    private static final String PROP_USER_AGENT                 = "User-Agent"                  ;

    // ----------------------------------------------------------------------------------------- Private Data

    private final String requestURL ;
    private final String charset    ;

    // ------------------------------------------------------------ Constructors
    /**
     *
     * @param requestURL must be non-null
     *
     */
    public SyncMLClientImpl(final String requestURL ,
                            final String charset    )
    throws IOException {

        if (requestURL == null) {
            throw new NullPointerException("request URL parameter is null");
        }

        this.requestURL = requestURL ;
        this.charset    = charset    ;
    }


    // ---------------------------------------------------------- Public methods

    /**
     *
     * @param request HTTP request
     * @return HTTP response
     */
    public String sendMessage(String request)
    throws IOException
    {

        HttpConnection c  = null;
        InputStream    is = null;
        OutputStream   os = null;

        StringBuffer b = null;
        byte[] data = null;

        try {

            long len;

            c = (HttpConnection)Connector.open(requestURL, Connector.READ_WRITE, true);

            setConfig(c, request.length());

            os = c.openOutputStream();
            os.write(request.getBytes());
            os.close();
            os = null;

            is = c.openInputStream();

            len = c.getLength();
            b = new StringBuffer(len >= 0 ? (int)len : 4096);
            if (len != -1) {
                // Read content-Length bytes, or until max is reached.
                int ch = 0;
                for (int i = 0; i < len; i++) {
                    if ((ch = is.read()) != -1 && ch != '\n') {

                        b.append((char) ch);

                    }
                }
            } else {
                // Read til the connection is closed, or til max is reached.
                // (Typical HTTP/1.0 script generated output)
                int ch = 0;
                len = 0;
                while ((ch = is.read()) != -1 && ch != '\n') {

                    b.append((char)ch);

                }
            }

            data = b.toString().getBytes();

        } finally {
            if (is != null)
                is.close();
            if (os != null)
                os.close();
            if (c != null)
                c.close();
        }

        if (charset != null && charset.length() > 0) {
            return new String(data, charset);
        } else {
            return new String(data);
        }

    }

    // ---------------------------------------------------------- Private methods

    /**
     * Add request properties for the configuration, profiles,
     * and locale of this system.
     * @param c current HttpConnection to receive user agent header
     */
    private void setConfig(HttpConnection c, int length) throws IOException {

        String conf    = System.getProperty(PROP_MICROEDITION_CONFIGURATION  );
        String prof    = System.getProperty(PROP_MICROEDITION_PROFILES       );
        String locale  = System.getProperty(PROP_MICROEDITION_LOCALE         );
        String ua      = "Profile/" + prof + " Configuration/" + conf         ;

        c.setRequestMethod(HttpConnection.POST);
        c.setRequestProperty(PROP_CONTENT_TYPE, "application/vnd.syncml+xml" );
        c.setRequestProperty(PROP_CONTENT_LENGTH, String.valueOf(length)     );
        c.setRequestProperty(PROP_USER_AGENT, ua                             );

        if (locale != null) {
            c.setRequestProperty(PROP_CONTENT_LANGUAGE, locale               );
        }

    }


}